#!/usr/bin/perl

#
# Licensed Materials - Property of IBM
# convert-testlog-xml.pl
# Copyright IBM Corporation 2012.
#
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#

if ($#ARGV != 1) {
	print "Syntax: convert <ocunit-test-log-input-file> <test-log-xml-output-file>\n";
	exit 1;
}

open (IN, "< $ARGV[0]") || die "Error opening input $ARGV[0]\n";
open (OUT, "> $ARGV[1]") || die "Error opening output $ARGV[1]\n";

$testSuiteName = "";
$readingTestData = 0;
$count = 0;
$failures = 0;
$totalTime = "";

while (<IN>) {
	chomp;

	if (/^Run test suite (.*)$/) {
		$testSuiteName = $1;
		$readingTestData = 1;
	}

	if (/^Test Case '(.*)' started.$/) {
		#print "Test Case\n";
		@entry = ();
		push @entry, $1;
		push @entry, "";
		push @results, [@entry];
	}
	elsif (/^Test Case '.*' failed \((.*) seconds\).$/) {
		#print "Test Case Fail\n";
		@entry = @{$results[$#results]};
		push @entry, "fail";
		push @entry, $1;
		$results[$#results] = [@entry];
	}
	elsif (/^Test Case '.*' passed \((.*) seconds\).$/) {
		#print "Test Case Pass\n";
		@entry = @{$results[$#results]};
		push @entry, "pass";
		push @entry, $1;
		$results[$#results] = [@entry];
	}
	elsif ($readingTestData == 1 && $#results >= 0) {
		@entry = @{$results[$#results]};
		#print "Entry> @entry $_\n";
		if ($#entry == 1) {
			#print "Entry size is correct\n";
			$entry[1] = $entry[1] . $_ . "\n";
			$results[$#results] = [@entry];
		}
	}

	if (/^Executed (.*) tests, with (.*) failures? \(.* unexpected\) in .* \((.*)\) seconds$/) {
		$readingTestData = 0;
		$count = $1;
		$failures = $2;
		$totalTime = $3;
	}
}

#print "C $count F $failures T $totalTime \n";

print OUT "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";
print OUT "<testsuite errors=\"0\" tests=\"$count\" time=\"$totalTime\" failures=\"$failures\" name=\"$testSuiteName\">\n";

for ($i = 0; $i <= $#results; $i++) {
	@entry = @{$results[$i]};
	#print "Entry:: @entry\n";
	if ($entry[2] eq "pass" || $entry[2] eq "fail") {
		$entry[0] =~ /-\[(\S+) (\S+)\]/;
		print OUT "\t<testcase time=\"$entry[3]\" classname=\"$1\" name=\"$2\">\n";

		if ($entry[2] eq "fail") {
			$entry[1] =~ s/\n/ /g;
			$entry[1] =~ s/&/&amp;/g;
			$entry[1] =~ s/</&lt;/g;
			$entry[1] =~ s/>/&gt;/g;
			$entry[1] =~ s/"/&quot;/g;
			print OUT "\t\t<failure type=\"Failure\" message=\"$entry[1]\">\n";
			print OUT "\t\t\t<system-out>\n";
			print OUT "$entry[1]\n";
			print OUT "\t\t\t</system-out>\n";
			print OUT "\t\t</failure>\n";
		}
		print OUT "\t</testcase>\n";
	}
}

print OUT "</testsuite>\n";

close IN;
close OUT;
